package de.kif.backend.repository

import de.kif.backend.database.DbPost
import de.kif.backend.database.dbQuery
import de.kif.backend.util.PushService
import de.kif.common.MessageType
import de.kif.common.Repository
import de.kif.common.RepositoryType
import de.kif.common.model.Post
import de.westermann.kobserve.event.EventHandler
import kotlinx.coroutines.runBlocking
import org.jetbrains.exposed.sql.*

object PostRepository : Repository<Post> {

    override val onCreate = EventHandler<Long>()
    override val onUpdate = EventHandler<Long>()
    override val onDelete = EventHandler<Long>()

    private fun rowToModel(row: ResultRow): Post {
        val id = row[DbPost.id]
        val name = row[DbPost.name]
        val content = row[DbPost.content]
        val url = row[DbPost.url]

        return Post(id, name, content, url)
    }

    override suspend fun get(id: Long): Post? {
        return dbQuery {
            rowToModel(DbPost.select { DbPost.id eq id }.firstOrNull() ?: return@dbQuery null)
        }
    }

    override suspend fun create(model: Post): Long {
        return dbQuery {
            val id = DbPost.insert {
                it[name] = model.name
                it[content] = model.content
                it[url] = model.url
            }[DbPost.id] ?: throw IllegalStateException("Cannot create model!")

            onCreate.emit(id)

            id
        }
    }

    override suspend fun update(model: Post) {
        if (model.id == null) throw IllegalStateException("Cannot update model which was not created!")
        dbQuery {
            DbPost.update({ DbPost.id eq model.id }) {
                it[name] = model.name
                it[content] = model.content
                it[url] = model.url
            }

            onUpdate.emit(model.id)
        }
    }

    override suspend fun delete(id: Long) {
        onDelete.emit(id)

        dbQuery {
            DbPost.deleteWhere { DbPost.id eq id }
        }
    }

    override suspend fun all(): List<Post> {
        return dbQuery {
            val result = DbPost.selectAll()

            result.map(this::rowToModel)
        }
    }

    suspend fun getByUrl(url: String): Post? {
        return dbQuery {
            val result = DbPost.select { DbPost.url eq url }

            runBlocking {
                result.firstOrNull()?.let {
                    rowToModel(it)
                }
            }
        }
    }

    fun registerPushService() {
        onCreate {
            runBlocking {
                PushService.notify(MessageType.CREATE, RepositoryType.POST, it)
            }
        }
        onUpdate {
            runBlocking {
                PushService.notify(MessageType.UPDATE, RepositoryType.POST, it)
            }
        }
        onDelete {
            runBlocking {
                PushService.notify(MessageType.DELETE, RepositoryType.POST, it)
            }
        }
    }
}
