package de.kif.backend.route

import de.kif.backend.authenticateOrRedirect
import de.kif.backend.isAuthenticated
import de.kif.backend.repository.PostRepository
import de.kif.backend.util.markdownToHtml
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.common.model.Permission
import de.kif.common.model.Post
import io.ktor.application.call
import io.ktor.html.respondHtmlTemplate
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.util.toMap
import kotlinx.html.*

fun Route.overview() {
    get("") {
        val user = isAuthenticated(Permission.POST)
        val editable = user != null

        val postList = PostRepository.all().asReversed()

        call.respondHtmlTemplate(MainTemplate()) {
            menuTemplate {
                this.user = user
                active = MenuTemplate.Tab.BOARD
            }
            content {
                div("overview") {
                    div("overview-main") {
                        if (editable) {
                            div("overview-new") {
                                a("post/new", classes = "form-btn") {
                                    +"New"
                                }
                            }
                        }

                        for (post in postList) {
                            div("overview-post post") {
                                span("post-name") {
                                    +post.name
                                }
                                if (editable) {
                                    a("/post/${post.id}", classes = "post-edit") {
                                        i("material-icons") { +"edit" }
                                    }
                                }
                                div("post-content") {
                                    unsafe {
                                        raw(markdownToHtml(post.content))
                                    }
                                }
                            }
                        }
                    }
                    div("overview-side") {
                        div("overview-shortcuts") {
                            a {
                                +"Wiki"
                            }
                            a {
                                +"Wiki"
                            }
                            a {
                                +"Wiki"
                            }
                            a {
                                +"Wiki"
                            }
                            a {
                                +"Wiki"
                            }
                            a {
                                +"Wiki"
                            }
                        }
                        div("overview-twitter") {
                            +"The Twitter Wall"
                        }
                    }
                }
            }
        }
    }


    get("/post/{id}") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@get
            val editPost = PostRepository.get(postId) ?: return@get
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.BOARD
                }
                content {
                    h1 { +"Edit post" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = editPost.name
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "url"
                                +"Url"
                            }
                            input(
                                name = "url",
                                classes = "form-control"
                            ) {
                                id = "places"
                                placeholder = "Places"
                                value = editPost.url
                            }
                        }

                        div("form-group") {
                            label {
                                htmlFor = "content"
                                +"Content"
                            }
                            textArea(rows = "10", classes = "form-control") {
                                name = "content"
                                id = "projector"

                                +editPost.content
                            }
                        }

                        div("form-group") {
                            a("/") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/post/${editPost.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post("/post/{id}") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@post
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            var post = PostRepository.get(postId) ?: return@post

            params["name"]?.let { post = post.copy(name = it) }
            params["url"]?.let { post = post.copy(url = it) }
            params["content"]?.let { post = post.copy(content = it) }

            PostRepository.update(post)

            call.respondRedirect("/")
        }
    }

    get("/post/new") {
        authenticateOrRedirect(Permission.POST) { user ->
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.BOARD
                }
                content {
                    h1 { +"Create post" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = ""
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "url"
                                +"Url"
                            }
                            input(
                                name = "url",
                                classes = "form-control"
                            ) {
                                id = "places"
                                placeholder = "Places"
                                value = Post.generateUrl()
                            }
                        }

                        div("form-group") {
                            label {
                                htmlFor = "content"
                                +"Content"
                            }
                            textArea(rows = "10", classes = "form-control") {
                                name = "content"
                                id = "projector"

                                +""
                            }
                        }

                        div("form-group") {
                            a("/") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post("/post/new") {
        authenticateOrRedirect(Permission.POST) { user ->
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val name = params["name"] ?: return@post
            val content = params["content"] ?: return@post
            val url = params["url"] ?: return@post

            val post = Post(null, name, content, url)

            PostRepository.create(post)

            call.respondRedirect("/")
        }
    }

    get("/post/{id}/delete") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@get

            PostRepository.delete(postId)

            call.respondRedirect("/")
        }
    }
}
